#!/usr/bin/perl
use XML::Simple;
use Data::Dumper;
use Image::Size;
use CGI;

#-------------------------------------------------------------------------------
# constants
#
my @blocked_servers = ('mondzeu.ch', '213.221.138.117');
my $pages_filespec = 'pages.xml';
my $icons_dir = 'Icons';
my $display_width = 321;
my $display_height = 462;
my $draw_grid = 0;

$separator = '-' x 80;
$indent = ' ' x 2;

#-------------------------------------------------------------------------------
# Input arguments
#
my $verbose = 1;

################################################################################
# Subroutines
#

#-------------------------------------------------------------------------------
# get description of a page from the XML description based on room and level
#
sub get_page_description {
  my ($pages_filespec, $room, $level, $icons_dir) = @_;
                                                             # read pages layout
  my $pages_ref = XMLin($pages_filespec);
#print Dumper($pages_ref);
                                                              # find proper page
  my $page_description;
  my $pages_ref = $pages_ref->{'page'};
  foreach my $page_ref (@$pages_ref) {
    if ( ($page_ref->{'room'} eq $room) and ($page_ref->{'level'} eq $level) ) {
      $page_description = $page_ref;
    }
  }
#print Dumper($page_description);
                                                            # complete page info
  my $buttons_ref = $page_description->{'buttons'};
#print Dumper($buttons_ref);
  foreach my $button (keys($buttons_ref)) {
#print "$button:\n";
    $buttons_ref->{$button}->{'icon'} = "$icons_dir/$buttons_ref->{$button}->{'icon'}.png";
    $buttons_ref->{$button}->{'room'} = $buttons_ref->{$button}->{'room'} || $room;
    $buttons_ref->{$button}->{'object'} = $buttons_ref->{$button}->{'object'} || $button;
#print Dumper($buttons_ref->{$button});
  }
#print Dumper($page_description);

  return(%$page_description);
}

#-------------------------------------------------------------------------------
# add SVG graphics in the HTML page
#
sub add_graphics {
  my ($indent, %graphics) = @_;
  my $indentLevel = 3;
  my $object_type = 'type';
#print Dumper(\%graphics);
                                                               # loop on buttons
  foreach my $vector (keys(%graphics)) {
#print Dumper($graphics{$vector});
    my $vector_ref = $graphics{$vector};
    print $indent x $indentLevel . "<$$vector_ref{$object_type}";
    foreach my $paramter (keys(%$vector_ref)) {
      if ($paramter ne $object_type) {
        print " $paramter='$$vector_ref{$paramter}'";
      }
    }
    print " />\n";
  }
}

#-------------------------------------------------------------------------------
# add buttons information to the page's CSS
#
sub add_buttons_CSS {
  my ($indent, %buttons) = @_;
  my $indentLevel = 3;
                                                               # loop on buttons
  foreach my $button (keys(%buttons)) {
#print Dumper($buttons{$button});
                                                                      # get info
    $button_info = $buttons{$button};
    my $type = $$button_info{'type'};
    my $icon_filespec = $$button_info{'icon'};
    my ($width, $height) = imgsize("$icon_filespec");
    $width = $$button_info{'width'} || $width;
    $info_height = $$button_info{'height'};
    $height = $info_height || $height;
    my $background_color = $$button_info{'color'};
                                                                     # write CSS
    print $indent x $indentLevel . "#$button {\n";
    $indentLevel = $indentLevel + 1;
    print $indent x $indentLevel . "left: $$button_info{'left'}px;\n";
    print $indent x $indentLevel . "top: $$button_info{'top'}px;\n";
    print $indent x $indentLevel . "width: ${width}px;\n";
    if ($type eq 'slider') {
      if ($height ne '') {
        print $indent x $indentLevel . "height: $height;\n";
      }
      print $indent x $indentLevel . "background-color: $background_color;\n";
    }
    else {
      print $indent x $indentLevel . "background-image: url($icon_filespec);\n";
      print $indent x $indentLevel . "height: ${height}px;\n";
    }
    $indentLevel = $indentLevel - 1;
    print $indent x $indentLevel . "}\n";
  }
}

#-------------------------------------------------------------------------------
# add buttons definitions in the HTML body
#
sub add_buttons_definitions {
  my ($indent, %buttons) = @_;
  my $indentLevel = 2;
                                                               # loop on buttons
  foreach my $button (keys(%buttons)) {
                                                                      # get info
    $button_info = $buttons{$button};
    my $link = $$button_info{'page'};
    if ($link ne '') {
      my ($room, $level) = split(/\//, $link);
      $link = "$0?room=$room&level=$level";
      $link =~ s/\A.*\///;
    }
    else {
      $link = $$button_info{'location'};
    }
                                                              # write definition
    print $indent x $indentLevel . '<button type="button" class="buttons"';
    print " id=\"$button\"";
    if ($link ne '') {
      if ($link =~ m/\Ahttp\:\/\//) {
        print " onclick=\"window.open('$link', '_blank')\"";
      }
      else {
        print " onclick=\"location.href='$link'\"";
      }
    }
    print " />\n";
  }
}

#-------------------------------------------------------------------------------
# add slider definitions in the HTML body
#
sub add_sliders_definitions {
  my ($indent, %buttons) = @_;
  my $indentLevel = 2;
                                                               # loop on buttons
  foreach my $button (keys(%buttons)) {
                                                                      # get info
    $button_info = $buttons{$button};
    my $type = $$button_info{'type'};
                                                              # write definition
    if ($type eq 'slider') {
      print $indent x $indentLevel . "<div class=\"slider\" id=\"$button\"></div>\n";
    }
  }
}

#-------------------------------------------------------------------------------
# add button handling functions in the javascript
#
sub add_button_handling_functions {
  my ($enable_controls, $server_name, %buttons) = @_;
                                                               # loop on buttons
  foreach my $button (keys(%buttons)) {
                                                                      # get info
    $button_info = $buttons{$button};
    my $type = $$button_info{'type'};
    my $room = $$button_info{'room'};
    my $kind = $$button_info{'kind'};
    my $object = $$button_info{'object'};
    my $min = $$button_info{'min'};
    my $max = $$button_info{'max'};
    my $step = $$button_info{'step'};
    my $value = $$button_info{'value'} || int( ($min + $max) / 2 );
    my $icon_on_filespec = $$button_info{'icon'};
    $icon_on_filespec =~ s/off\.png\Z/On.png/i;
    my $icon_off_filespec = $$button_info{'icon'};
    $icon_off_filespec =~ s/on\.png\Z/Off.png/i;
    my $send_command = '// served by ' . $server_name;
    if ($enable_controls) {
      $send_command = '$.post(\'sendState.cgi\', {room: room, kind: kind, object: object, value: value})';
    }
    if ($type eq 'slider') {
      $send_command =~ s/value: value/value: ui.value/;
    }
                                                                # write function
    if ($type eq 'on_off') {
print <<"EOT";

        \$('#$button').on('click', function() {
          room = '$room';
          kind = '$kind';
          object = '$object';
          if (\$(this).attr('on_off') == 1) {
            \$(this).attr('on_off', 0);
            \$(this).css('background-image', 'url($icon_off_filespec)');
            value = 'off';
          }
          else {
            \$(this).attr('on_off', 1);
            \$(this).css('background-image', 'url($icon_on_filespec)');
            value = 'on';
          }
          $send_command
          console.log('sending ' + room + ' ' + kind + ' ' + object + ' ' + value);
        });
EOT
    }
    if ($type eq 'press') {
print <<"EOT";
        \$('#$button').on('click', function() {
          room = '$room';
          kind = '$kind';
          object = '$object';
          value = '$value';
          $send_command
          console.log('sending ' + room + ' ' + kind + ' ' + object + ' ' + value);
        });
EOT
    }
    if ($type eq 'slider') {
print <<"EOT";
        \$("#$button").slider({
          min        : $min,
          max        : $max,
          step       : $step,
          value      : $value,
          orientation: "horizontal",
          change     : function(event, ui) {
            room = '$room';
            kind = '$kind';
            object = '$object';
            $send_command
            console.log('sending ' + room + ' ' + kind + ' ' + object + ' ' + ui.value);
          }
        });
EOT
    }
  }
}

################################################################################
# Main program
#
                                                   # get environment information
my $cgi = CGI->new;
my $server_name = $cgi->server_name;
my $enable_controls = 1;
foreach my $server (@blocked_servers) {
  if ($server_name =~ m/\A$server/i) {
    $enable_controls = 0;
  }
}
                                                              # get pages layout
my $room = $cgi->param('room') || 'apartment';
my $level = $cgi->param('level') || 'rooms';
my %page_description = get_page_description(
  $pages_filespec,
  $room, $level,
  $icons_dir
);
my $buttons_ref = $page_description{'buttons'};
my $graphics_ref = $page_description{'graphics'};
                                                             # start HTML header
print $cgi->header;
print <<"EOT";
<html>
  <head>
    <title>$room $level control</title>
    <meta name="viewport" content="width=device-width, initial-scale=1.0, maximum-scale=1.0">
    <link href="jquery-ui.min.css" rel="stylesheet">
    <style type="text/css">
      html, body, form, fieldset, p, div, h1, h2, h3, h4, h5, h6 {
        margin:0;
        padding:0;
        -webkit-text-size-adjust:none;
      }
      body {
        background-color: #$page_description{'background'};
      }
      a {
        text-decoration:none;
      }
      button.buttons {
        position: absolute;
        margin:0;
        border: none;
        background-color: transparent;
        background-repeat: no-repeat;
        cursor: pointer;
      }
      .slider {
        position: absolute;
        margin: 0px;
        height: 5px;
        border-width: 2px;
        border-color: #333333 #777777 #777777 #333333;
        border-radius: 4px;
      }
      .slider .ui-slider-handle {
        top: -8px;
        height: 20px;
        width: 10px;
        background-color: #aaaaaa;
        border-width: 2px;
        border-color: #dddddd #333333 #333333 #dddddd;
        border-radius: 0 0 5px 5px;
      }
EOT
                                                               # add buttons CSS
add_buttons_CSS($indent, %$buttons_ref);
                                                         # terminate HTML header
my $svg_width = $display_width + 1;
my $svg_height = $display_height + 1;
print <<"EOT";
    </style>
  </head>
  <body>
    <!-- SVG graphics ...................................................... -->

    <svg id="svg" width="$svg_width" height="$svg_height" position: absolute; left: 0px; top: 0px;>
EOT
#     <rect x="0" y="230" rx="10" ry="10" width="120" height="80" fill="#FFFFBF" />
#     <rect x="140" y="230" rx="10" ry="10" width="120" height="80" fill="#FFE79F" />
                                                                     # draw grid
if ($draw_grid) {
  print $indent x 3 . "<rect x=\"0\" y=\"0\" width=\"$svg_width\" height=\"$svg_height\" fill=\"none\" stroke=\"black\" stroke-width=\"1\" />\n";
  my %grid = (10 => 0.5, 50 => 0.75, 100 => 2);
  foreach my $step (keys(%grid)) {
    for my $index (1..$display_width/$step) {
      my $x_pos = $step*$index;
      print $indent x 3 . "<line x1=\"$x_pos\" y1=\"0\" x2=\"$x_pos\" y2=\"$display_height\" stroke-width=\"$grid{$step}\" stroke=\"black\" />\n"
    }
  }
  foreach my $step (keys(%grid)) {
    for my $index (1..$display_height/$step) {
      my $y_pos = $step*$index;
      print $indent x 3 . "<line x1=\"0\" y1=\"$y_pos\" x2=\"$display_width\" y2=\"$y_pos\" stroke-width=\"$grid{$step}\" stroke=\"black\" />\n"
    }
  }
}
                                                                 # draw graphics
add_graphics($indent, %$graphics_ref);
                                                    # prepare slider definitions
print <<"EOT";
    </svg>

    <!-- Sliders definition ................................................ -->

EOT
                                                       # add sliders definitions
add_sliders_definitions($indent, %$buttons_ref);
                                                   # prepare buttons definitions
print <<"EOT";

    <!-- Buttons definition ................................................ -->

EOT
                                                       # add buttons definitions
add_buttons_definitions($indent, %$buttons_ref);
                                                  # start button handling script
print <<"EOT";

    <!-- Button handling javascript ........................................ -->

    <script src="jquery-2.1.4.min.js"></script>
    <script src="jquery-ui.min.js"></script>
    <script>
      \$().ready(function() {
        var room = '';
        var kind = '';
        var object = '';
        var value = '';
EOT
                                                 # add button handling functions
add_button_handling_functions($enable_controls, $server_name, %$buttons_ref);
                                                                 # end HTML page
print <<"EOT";
      });
    </script>

  </body>
</html>
EOT
